/*

DESCRIPTION:
    Main include file for sine wave generator for true-sine inverter based on
    PIC16F18324.

SYNTAX:  n/a

KNOWN BUGS:  n/a.

COPYRIGHT:  (c) 2017, Brian Cornell, all rights reserved.

DEPENDENCIES:  none

VERSION:  1.0, development release

MODIFICATION HISTORY:

    1 Nov  17, BWC, created file.


*/



//#define _DEBUG                              //Uncomment for conditional testing
//#define _CFGFB                              //Full bridge config
#define _CFGHB                              //Half bridge config


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Version constants & definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define VERMAJOR                1
#define VERMINOR                0
#define VERBUILD                8


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * System wide constants & definitions.  Convention:  S_<TYPE>_<INFO>
 * Types:
 *  + DC=Duty Cycle
 *  + EN=ENable bit
 *  + RT=RaTio of a voltage presented to PIC, internal or external
 *  + LM=LiMiting operational value
 *  + TM=TiMing
 *  + MP=Peripheral port bit MaPping
 *  + QY=quantity (count) of something
 *  + CD=command data
 *  + BL=boolean value
 * 
 * Info is user defined and should be descriptive.
 * 
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define S_QY_PWMQSCYC           100     //PWM cycles in quarter sine
#define S_LM_PWMINDEX           (S_QY_PWMQSCYC - 1) //PWM index limit
#define S_CD_RUN                1       //Sine wave generator start
#define S_CD_STOP               0       //Sine wave generator stop
#define S_BL_INCR               1       //Sine wave increasing
#define S_BL_DECR               0       //Sine wave decreasing
#define S_BL_POS                0       //Sine positive half cycle
#define S_BL_NEG                1       //Sine negative half cycle
#define S_TM_INT                100     //#mS to wait after initialization
#define S_LM_TMR2START          3       //# of periods to wait before enabling CWG
#define S_BL_HSON               PWM5OUT //TRUE if PWM output is high


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                         Object sizes & positions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SIZ_INT_ADC             2       //# bytes in ADC integer
#define POS_HIG_ADC             1       //ADC result high byte
#define POS_LOW_ADC             0       //ADC result low byte


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Analog channel mapping:  channel=port=pin.  The values assigned to these
 * constants should match those used by ADC and bit-aligned for direct loading
 * into ADC control register.  Channels available depend on specific chip.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define ANLCAN0                 0x00    //ANA0=RA0=13
#define ANLCAN1                 0x04    //ANA1=RA1=12
#define ANLCAN2                 0x08    //ANA2=RA2=11
#define ANLCAN3                 0x0C    //RESERVED
#define ANLCAN4                 0x10    //ANA4=RA4=3
#define ANLCAN5                 0x14    //ANA5=RA5=2
#define ANLCAN6                 0x30    //ANB4=n/a
#define ANLCAN7                 0x34    //ANB5=n/a
#define ANLCAN8                 0x38    //ANB6=n/a
#define ANLCAN9                 0x3C    //ANB7=n/a
#define ANLCAN10                0x40    //ANC0=RC0=10
#define ANLCAN11                0x44    //ANC1=RC1=9
#define ANLCAN12                0x48    //ANC2=RC2=8
#define ANLCAN13                0x4C    //ANC3=RC3=7
#define ANLCAN14                0x50    //ANC4=RC4=6
#define ANLCAN15                0x54    //ANC5=RC5=5
#define ANLCAN16                0x58    //ANC6=n/a
#define ANLCAN17                0x5C    //ANC7=n/a


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                           PPS & direct pin mapping
 * 
 * For PPS output pin assignments both aliases for pin registers and constants 
 * mapping what peripheral connects to the pin.
 *      Pin register alias convention:  PPS_P_XXX_YY where
 *          XXX=peripheral name
 *          YY=channel or other detail; can be omitted
 *      Peripheral mapping convention:  PPS_M_XXX_YY same as alias
 * 
 * Input pins only require mapping constant.
 * 
 * For pins accessed directly or assigned in a peripheral's register (like ADC)
 * use the following convention:  PIN_X_YYYYYY where
 *          X=Input or Output
 *          YYYYYY=user assigned mnemonic describing function
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef _CFGFB
#define PPS_P_CWG1_A            RC1PPS  //A output on pin 9
#define PPS_M_CWG1_A            0x08    //PPS value for CWG1 A signal
#define PPS_P_CWG1_D            RC2PPS  //D output on pin 8
#define PPS_M_CWG1_D            0x0B    //PPS value for CWG1 D signal
#define PPS_P_CWG1_C            RC4PPS  //C output on pin 6
#define PPS_M_CWG1_C            0x0A    //PPS value for CWG1 C signal
#define PPS_P_CWG1_B            RC3PPS  //B output on pin 7
#define PPS_M_CWG1_B            0x09    //PPS value for CWG1 B signal
#define PPS_M_CLC_0             0x11    //RC1
#define PPS_M_CLC_1             0x14    //RC4
#define PPS_M_CLC_2             0x12    //RC2
#define PPS_M_CLC_3             0x13    //RC3
#define PPS_P_CLC1              RC0PPS  //CLC1 output on pin 10
#define PPS_M_CLC1              0x04    //PPS value for CLC1
#define PPS_P_CLC2              RC5PPS  //CLC2 output on pin 5
#define PPS_M_CLC2              0x05    //PPS value for CLC2
#endif

#ifdef _CFGHB
#define PPS_P_CWG1_A            RA2PPS  //Output on pin 11
#define PPS_M_CWG1_A            0x08    //PPS mapping to peripheral
#define PPS_P_CWG1_B            RA4PPS  //Output on pin 3
#define PPS_M_CWG1_B            0x09    //PPS mapping to peripheral

#define PPS_M_CLC_0             0x02    //RA2, pin 11
#define PPS_M_CLC_1             0x04    //RA4, pin 3
#define PPS_M_CLC_2             0x10    //RC0, pin 10
#define PPS_M_CLC_3             0x15    //RC5, pin 5

#define PPS_P_CLC1              RC2PPS  //Output on pin 8 (HIN-A)
#define PPS_M_CLC1              0x04    //PPS mapping to peripheral
#define PPS_P_CLC2              RC1PPS  //Output on pin 9 (LIN-A)
#define PPS_M_CLC2              0x05    //PPS mapping to peripheral
#define PPS_P_CLC3              RC3PPS  //Output on pin 7 (HIN-B)
#define PPS_M_CLC3              0x06    //PPS mapping to peripheral
#define PPS_P_CLC4              RC4PPS  //Output on pin 6 (LIN-B)
#define PPS_M_CLC4              0x07    //PPS mapping to peripheral

#define PIN_O_ENA               RC0     //Enables A high-side and B low-side
#define PIN_O_ENB               RC5     //Enables B high-side and A low-side
#endif

#define PIN_I_RUN               RA5     //Commands sine wave start/stop


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Port/pin configuration:  I/O, analog/digital, slew rate, etc.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef _CFGFB
//Port I/O:  1=input, 0=output
#define IO_PORTA                0b11111111
#define IO_PORTB                0b11111111
#define IO_PORTC                0b11000000
//Port Analog/Digital:  1=analog, 0=digital
#define AD_PORTA                0b11011111
#define AD_PORTB                0b11111111
#define AD_PORTC                0b11000000
#endif

#ifdef _CFGHB
//Port I/O:  1=input, 0=output
#define IO_PORTA                0b11101011
#define IO_PORTB                0b11111111
#define IO_PORTC                0b11000000
//Port Analog/Digital:  1=analog, 0=digital
#define AD_PORTA                0b11001011
#define AD_PORTB                0b11111111
#define AD_PORTC                0b11000000
#endif

//Port weak pull-up:  1=enabled, 0=disabled
#define WP_PORTA                0b00000000
#define WP_PORTB                0b00000000
#define WP_PORTC                0b00000000
//Open drain:  1=open drain, 0=push-pull
#define OD_PORTA                0b00000000
#define OD_PORTB                0b00000000
#define OD_PORTC                0b00000000
//Slew rate:  1=limited, 0=maximum
#define SR_PORTA                0b00000000
#define SR_PORTB                0b00000000
#define SR_PORTC                0b00000000
//Input level:  1=ST, 0=TLL
#define IL_PORTA                0b00000000
#define IL_PORTB                0b00000000
#define IL_PORTC                0b00000000
//Port values to initially set output latches to
#define VL_PORTA                0b00000000
#define VL_PORTB                0b00000000
#define VL_PORTC                0b00000000

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                            Macro functions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * ADC configuration & acquisition constants:  NOT CONFIGURED
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//AD acquisition configuration & control constants
#define CFG_ADC_CON0            0x00
#define CFG_ADC_CON1            0x00
#define CFG_ADC_ACT             0x00

//Internal channels (peripherals)
#define ADC_CH_VSS              0xF0    //Vss
#define ADC_CH_TMP              0xF4    //Temperature indicator
#define ADC_CH_DAC_1            0xF8    //DAC1
#define ADC_CH_FVR              0xFC    //FVR


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure system clock.  Expects 1MHz HFINTOSC to be active at boot.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_OSC_SRC     0x0             //Nibble setting oscillator selection
#define CFG_OSC_DIV     0x0             //Nibble setting divider selection (1)
//#define CFG_OSC_FRQ     0x6             //Nibble setting HFINTOSC freququency (16MHz)
#define CFG_OSC_FRQ     0x5             //Nibble setting HFINTOSC freququency (24MHz)
//#define _XTAL_FREQ      16000000        //Must match Fosc to use __delayxxx macros
#define CFG_OSC_OSCTUNE 0x00            //Adjustment to HFINTOSC to obtain 60Hz

#define _XTAL_FREQ      24000000        //Must match Fosc to use __delayxxx macros


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FVR & DAC configuration constants
 * FVR:  not configured
 * DAC1:  not configured
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_FVR_C               0x00
#define FI_FVR_DLY              30      //Time delay in uS for FVR buffer to stabilize

//DAC1:  
#define CFG_DAC_1_C             0x00
#define CFG_DAC_1_V             0x00


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Timer configuration constants
 * TMR2:  PWM5 time base interrupt for loading successive DC
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Timer2
#define CFG_TMR_2_CON           0x00    //No pre/post scale, off
//#define CFG_TMR_2_FRQ           164     //24,242.4242Hz
#define CFG_TMR_2_FRQ           249     //24kHz


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * PWM configuration constants
 * PWM5:  digital sine wave modulation
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//PWM5
#define CFG_PWM_5_CON           0x00    //Disabled, active high output
#define CFG_PWM_5_TMRS          0x05    //TMR2 is period source
#define CFG_PWM_DCH_BITS        2
#define CFG_PWM_DCL_BITS        6
#define CFG_PWM_DCL_MASK        0x0003  


#ifdef _CFGFB
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Complementary waveform generator configuration constants
 * CWG1:  full bridge steering of PWM5 output
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//CWG1
#define CFG_CWG_1_CON0          0x02    //Disabled, full-bridge forward mode
#define CFG_CWG_1_CON1          0x00    //Normal polarity on all outputs
#define CFG_CWG_1_CLK           0x00    //Fosc is clock
#define CFG_CWG_1_DAT           0x07    //PWM5 is data input signal
#define CFG_CWG_1_AS0           0xA8    //Auto-shutdown ON, outputs use inactive state
#define CFG_CWG_1_AS1           0x00    //No HW auto-shutdown sources
#define CFG_CWG_1_DBR           0x00    //No rising dead-band delay
#define CFG_CWG_1_DBF           0x00    //No falling dead-band delay


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC1 as OR-XOR logic cell to control A low side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_1_CON       0x01    //Not enabled, no interrupts, OR-XOR mode
#define CFG_CLC_1_POL       0x00    //No inverted polarities
#define CFG_CLC_1_SEL0      0x00    //CLCIN0PPS (A low side)
#define CFG_CLC_1_SEL1      0x01    //CLCIN1PPS (B low side)
#define CFG_CLC_1_SEL2      0x02    //CLCIN2PPS (A high side)
#define CFG_CLC_1_SEL3      0x02    //CLCIN2PPS (A high side)
#define CFG_CLC_1_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_1_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_1_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_1_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC2 as OR-XOR logic cell to control B low side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_2_CON       0x01    //Not enabled, no interrupts, OR-XOR mode
#define CFG_CLC_2_POL       0x00    //No inverted polarities
#define CFG_CLC_2_SEL0      0x00    //CLCIN0PPS (A low side)
#define CFG_CLC_2_SEL1      0x01    //CLCIN1PPS (B low side)
#define CFG_CLC_2_SEL2      0x03    //CLCIN3PPS (B high side)
#define CFG_CLC_2_SEL3      0x03    //CLCIN3PPS (B high side)
#define CFG_CLC_2_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_2_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_2_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_2_LS_3      0x80    //d4T is gated into g4

#endif

#ifdef _CFGHB
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Complementary waveform generator configuration constants
 * CWG1:  full bridge steering of PWM5 output
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//CWG1
#define CFG_CWG_1_CON0          0x04    //Disabled, half-bridge mode
#define CFG_CWG_1_CON1          0x00    //Normal polarity on all outputs
#define CFG_CWG_1_CLK           0x00    //Fosc is clock
#define CFG_CWG_1_DAT           0x07    //PWM5 is data input signal
#define CFG_CWG_1_AS0           0xA8    //Auto-shutdown ON, outputs use inactive state
#define CFG_CWG_1_AS1           0x00    //No HW auto-shutdown sources
#define CFG_CWG_1_DBR           0x02    //Rising dead-band delay
#define CFG_CWG_1_DBF           0x02    //Falling dead-band delay


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC1 as AND logic cell to control A high-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_1_CON       0x02    //Not enabled, no interrupts, AND mode
#define CFG_CLC_1_POL       0x00    //No inverted polarities
#define CFG_CLC_1_SEL0      0x00    //CLCIN0PPS
#define CFG_CLC_1_SEL1      0x00    //CLCIN0PPS
#define CFG_CLC_1_SEL2      0x02    //CLCIN2PPS
#define CFG_CLC_1_SEL3      0x02    //CLCIN2PPS
#define CFG_CLC_1_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_1_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_1_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_1_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC2 as AND-OR logic cell to control A low-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_2_CON       0x00    //Not enabled, no interrupts, AND-OR mode
#define CFG_CLC_2_POL       0x00    //No inverted polarities
#define CFG_CLC_2_SEL0      0x01    //CLCIN1PPS
#define CFG_CLC_2_SEL1      0x01    //CLCIN1PPS
#define CFG_CLC_2_SEL2      0x03    //CLCIN3PPS
#define CFG_CLC_2_SEL3      0x03    //CLCIN3PPS
#define CFG_CLC_2_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_2_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_2_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_2_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC3 as AND logic cell to control B high-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_3_CON       0x02    //Not enabled, no interrupts, AND mode
#define CFG_CLC_3_POL       0x00    //No inverted polarities
#define CFG_CLC_3_SEL0      0x00    //CLCIN0PPS
#define CFG_CLC_3_SEL1      0x00    //CLCIN0PPS
#define CFG_CLC_3_SEL2      0x03    //CLCIN3PPS
#define CFG_CLC_3_SEL3      0x03    //CLCIN3PPS
#define CFG_CLC_3_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_3_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_3_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_3_LS_3      0x80    //d4T is gated into g4

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC4 as AND-OR logic cell to control B low-side.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_4_CON       0x00    //Not enabled, no interrupts, AND-OR mode
#define CFG_CLC_4_POL       0x00    //No inverted polarities
#define CFG_CLC_4_SEL0      0x01    //CLCIN1PPS
#define CFG_CLC_4_SEL1      0x01    //CLCIN1PPS
#define CFG_CLC_4_SEL2      0x02    //CLCIN2PPS
#define CFG_CLC_4_SEL3      0x02    //CLCIN2PPS
#define CFG_CLC_4_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_4_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_4_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_4_LS_3      0x80    //d4T is gated into g4

#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Object definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Object to store a measured voltage with 3-digit mV resolution
typedef struct {
    unsigned int vmv;                   //Volts in 10K/1K places, mV in 100/10/1
    unsigned int volt;                  //Volts as integer
    unsigned int mv;                    //mV as integer 
} volts;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                             Status codes
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define STS_BOOT        0x00            //Startup
#define STS_RUN         0x01            //Running
#define STS_BUS_UVLO    0x02            //Stopped - bus under-volt lockout
#define STS_CHL_OCLO    0x03            //Stopped - over-current lockout
#define STS_OUT_UVLO    0x04            //Stopped - output under-volt lockout
#define STS_OUT_OVLO    0x05            //Stopped - output over-volt lockout
#define STS_BUS_OVLO    0x06            //Disabled - bus over-volt lockout

#define STS_CODE_ERR    0x02            //Codes < this value are not errors


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Function declarations.  See source file for explanation of each function.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void interrupt InterruptExec(void);
void Initialize(void);
void main(void);


//End of file